#!/bin/bash
# Variablen:
export INSTALLER_DNS="tu-dresden.de"
export INSTALLER_CONFIG=''
export INSTALLER_CONFIG_FILE="/tmp/installer-tmpfile.sh"
export INSTALLER_CONFIG_URL=''
export INSTALLER_DEBUGGING=''
export INSTALLER_ERROR_ACTION=reboot
export INSTALLER_LOGFILE="/tmp/installer.log"
export INSTALLER_MODE='sdm_install'
export INSTALLER_NOTIFY_URL=''

# VERWENDETE VARIABLEN
# OS_DISTRIBUTION

# ----------------------------------------------------------------------------------------------------------------------
# Vorbereitung
update-ca-certificates
uname -r
[ -d /sys/firmware/efi ] && echo UEFI || echo BIOS
sleep 5
export PATH=/usr/local/bin:/usr/local/sbin:/usr/bin:/usr/sbin:/bin:/sbin/



# damit das Skript von überall aus gestartet werden kann:
# in eigenen Ordner wechseln
# dirname: gibt Pfad aus ohne Dateiname
# readlink: kompletter Pfad aus $0
cd "$(dirname "$(readlink -f "$0")")"

. ./functions.sh

# prüfe, wenn uid ungliche Null
if ! [ $EUID -eq 0 ]
then
    echo "Der SDM-Installer muss als Benutzer »root« gestartet werden!"
    exit 1
fi

clear
# dmesg -n: gibt auf Konsole Nr x aus
dmesg -n 1
# swap auschalten
swapoff -a


# ----------------------------------------------------------------------------------------------------------------------
# Auf Netzwerk warten

# hier wird mit Status-Anzeige 10x probiert INSTALLER_DNS (tu-dresden.de) aufzulösen (um die DNS-Konfiguration durch DHCP abzuwarten)
(

counter=0
while ! nslookup $INSTALLER_DNS 2> /dev/null > /dev/null
do
    [ "$counter" -ge 100 ] && exit 1
    
    counter=$(expr $counter + 1)
    echo $counter
    
    sleep 1
done

echo XXX
echo 100
echo "Netzwerkverbindung erfolgreich."
echo XXX

sleep 5
# Lampda Funktion, die Klammern "fangen" die Ausgabe ab
# --gauge Infobox mit Höhe Breite %Anzeige, XXX ist für dialog erforderlich
) | dialog --title "SDM-Installation" --gauge "Warte auf Netzwerk-Verbindung ..." 15 70 0

if [ ${PIPESTATUS[0]} -eq 1 ]
then
    dialog_info "Netzwerkverbindung gescheitert! Der Rechner wird neugestartet." && sleep 3
    $INSTALLER_ERROR_ACTION; exit 1
fi


export hostname="$(hostname)"


# ----------------------------------------------------------------------------------------------------------------------
# Parameter

# Kernel-Parameter in Umgebung übernehmen
export $(cat /proc/cmdline)

# Debug-Modus -> Installation überspringen und in eine Shell droppen
# verlassen durch? vielleicht reboot oder poweroff
if [ -n "$INSTALLER_DEBUGGING" ]
then
    while true
    do
        bash
    done
fi

# Parameter prüfen
if [ -z "$OS_DISTRIBUTION" ] && [ -n "$INSTALLER_MODE" ]
then
    dialog_info "Fehlende Parameter! Der Rechner wird neugestartet." && sleep 3
    $INSTALLER_ERROR_ACTION; exit 1
fi

# ----------------------------------------------------------------------------------------------------------------------
# Konfiguration laden
if [ -n "$INSTALLER_CONFIG_URL" ]
then
    if [[ "$INSTALLER_CONFIG_URL" == http* ]]
    then
        INSTALLER_CONFIG_URL="$INSTALLER_CONFIG_URL?hostname=$hostname"
        dialog_info "Hole Installationskonfiguration von Server ..." && sleep 1
        
        if ! curl -s --capath ./certs/ -o "$INSTALLER_CONFIG_FILE" "$INSTALLER_CONFIG_URL?hostname=$hostname" > /dev/null
        then
            dialog_info "Installations-Skript konnten nicht geholt werden!\n\nDer Rechner wird neugestartet." && sleep 3
            $INSTALLER_ERROR_ACTION; exit 1
        fi
        chmod +x "$INSTALLER_CONFIG_FILE"
        export $INSTALLER_CONFIG="$INSTALLER_CONFIG_FILE"
    fi
fi


# Konfiguration entweder über URL oder fest in das Tinycore einbauen unter $INSTALLER_CONFIG_FILE
# das . VAR lädt den Inhalt der Datei in die Shell Umgebung
. "$INSTALLER_CONFIG"


# nochmal aus cmdline überschreiben
export $(cat /proc/cmdline)

if [ $(type -t $INSTALLER_MODE) != "function" ]
then
    dialog_info "Unbekannte Methode: $INSTALLER_MODE. Der Rechner wird neugestartet." && sleep 3
    $INSTALLER_ERROR_ACTION; exit 1
fi

[ -n "$INSTALLER_LOGFILE_URL" ] && INSTALLER_LOGFILE_URL="$INSTALLER_LOGFILE_URL?hostname=$hostname"
[ -n "$INSTALLER_NOTIFY_URL" ] && INSTALLER_NOTIFY_URL="$INSTALLER_NOTIFY_URL?hostname=$hostname"


# ----------------------------------------------------------------------------------------------------------------------


dialog_info "Starte Installation ... \n\nHostname: $hostname \nDistribution: $OS_DISTRIBUTION\nModus: $INSTALLER_MODE\nKonfiguration: $INSTALLER_CONFIG"
sleep 5

# Installation aufrufen
($INSTALLER_MODE)

if [ $? -eq 0 ]
then
    dialog_info "Installationsprotokolle werden hochgeladen ..."
    upload_logfile
    
    if [ -n "$INSTALLER_NOTIFY_URL" ]
    then
        dialog_info "Installation erfolgreich.\n\nMelde an Server ..."
        
        if curl -s --capath ./certs/ "$INSTALLER_NOTIFY_URL" > /dev/null
        then
            dialog_info "Installation erfolgreich.\n\nDer Rechner wird neugestartet." && sleep 3
        else
            dialog_msg "Installation erfolgreich.\n\nServer nicht erreichbar.\n\nBestätigen, um Rechner neuzustarten."
        fi
    else
        dialog_msg "Installation erfolgreich.\n\nBestätigen, um Rechner neuzustarten."
    fi
fi

#reboot
exit 0
